import dataclasses
import pathlib
import sys
import unittest
from typing import Literal

from odoo_runbot.runbot_config import RunbotExcludeWarning, RunbotToolConfig

if sys.version_info >= (3, 11):
    import tomllib
else:
    import tomli as tomllib
from . import config_tester


class TestConfigLoader(unittest.TestCase):
    maxDiff = None
    F_FTYPE = Literal["pyproject", "custom"]

    def assertConfigEqual(self, config: RunbotToolConfig, config_expected: RunbotToolConfig):  # noqa : N802
        self.assertDictEqual(dataclasses.asdict(config), dataclasses.asdict(config_expected))

    def sample_config(self, fname: str, f_type: F_FTYPE) -> RunbotToolConfig:
        path = pathlib.Path(__file__).resolve().parent.joinpath("sample_config", f"{f_type}_{fname}")
        with path.open(mode="rb") as pyproject_toml:
            data = tomllib.load(pyproject_toml)
        if f_type == "pyproject":
            return RunbotToolConfig.load_from_toml_data(data.get("tool", {}).get("runbot"), config_file=path)
        return RunbotToolConfig.load_from_toml_data(data, config_file=path)

    def test_minimal_config(self):
        """[tool.runbot]
        modules = ["module_to_test"]
        """
        self.assertConfigEqual(
            self.sample_config("minimal.toml", "pyproject"), self.sample_config("minimal.toml", "custom")
        )
        self.assertConfigEqual(self.sample_config("minimal.toml", "pyproject"), config_tester.minimal_config_test())
        self.assertConfigEqual(self.sample_config("minimal.toml", "custom"), config_tester.minimal_config_test())

    def test_minimal_file_config(self):
        """[tool.runbot]
        modules = ["module_to_test"]
        """
        self.assertConfigEqual(
            self.sample_config("module.file.toml", "pyproject"), self.sample_config("module.file.toml", "custom")
        )

        self.assertConfigEqual(
            self.sample_config("module.file.toml", "pyproject"), config_tester.classic_file_config_test()
        )
        self.assertConfigEqual(
            self.sample_config("module.file.toml", "custom"), config_tester.classic_file_config_test()
        )

    def test_pyproject_classic(self):
        self.assertConfigEqual(
            self.sample_config("classic.toml", "pyproject"), self.sample_config("classic.toml", "custom")
        )
        self.assertConfigEqual(
            self.sample_config("classic.full.toml", "pyproject"), self.sample_config("classic.full.toml", "custom")
        )
        self.assertConfigEqual(
            self.sample_config("classic.toml", "pyproject"), self.sample_config("classic.full.toml", "pyproject")
        )

        self.assertConfigEqual(
            self.sample_config("classic.toml", "custom"), self.sample_config("classic.full.toml", "custom")
        )
        self.assertConfigEqual(
            self.sample_config("classic.full.toml", "pyproject"), config_tester.pyproject_classic_test()
        )
        self.assertConfigEqual(self.sample_config("classic.toml", "pyproject"), config_tester.pyproject_classic_test())
        self.assertConfigEqual(self.sample_config("classic.toml", "custom"), config_tester.pyproject_classic_test())
        self.assertConfigEqual(
            self.sample_config("classic.full.toml", "custom"), config_tester.pyproject_classic_test()
        )
        self.assertConfigEqual(
            self.sample_config("classic.toml", "pyproject"), self.sample_config("classic.full.toml", "custom")
        )
        self.assertConfigEqual(self.sample_config("classic.toml", "pyproject"), config_tester.pyproject_classic_test())
        self.assertConfigEqual(self.sample_config("classic.toml", "custom"), config_tester.pyproject_classic_test())

    def test_pyproject_complex(self):
        self.assertConfigEqual(
            self.sample_config("complex.toml", "pyproject"), self.sample_config("complex.toml", "custom")
        )
        self.assertConfigEqual(self.sample_config("complex.toml", "pyproject"), config_tester.pyproject_complex_test())
        self.assertConfigEqual(self.sample_config("complex.toml", "custom"), config_tester.pyproject_complex_test())

    def test_min_max_match_log_filter(self):
        assert RunbotExcludeWarning(name="A", regex="A", min_match=2) == RunbotExcludeWarning(
            name="A",
            regex="A",
            min_match=2,
            max_match=2,
        ), "Assert Min and max match follow each other if not set"
        assert RunbotExcludeWarning(name="A", regex="A", min_match=10, max_match=2) == RunbotExcludeWarning(
            name="A",
            regex="A",
            max_match=10,
            min_match=10,
        ), "Assert Min and max match follow each other if not set"
        assert RunbotExcludeWarning(name="A", regex="A", min_match=-1) == RunbotExcludeWarning(
            name="A",
            regex="A",
            max_match=1,
            min_match=0,
        ), "Assert if Min is -1 then this means 0 min match"
        assert RunbotExcludeWarning(name="A", regex="A", min_match=0) == RunbotExcludeWarning(
            name="A",
            regex="A",
            max_match=1,
            min_match=0,
        ), "Assert if Min is 0 then this means 0 min match"
        assert RunbotExcludeWarning(name="A", regex="A", max_match=0) == RunbotExcludeWarning(
            name="A",
            regex="A",
            max_match=0,
            min_match=0,
        ), "Assert if Max is 0 means exacly 0 match possible"
        assert RunbotExcludeWarning(name="A", regex="A", max_match=999) == RunbotExcludeWarning(
            name="A",
            regex="A",
            max_match=100,
            min_match=1,
        ), "Assert if Max can't be more than 100If you want more than 100, you should fix this logger :-)"



from typer.testing import CliRunner

from odoo_runbot.cli import app

runner = CliRunner()


def test_app():
    result = runner.invoke(app, ["--workdir", "odoo_project", "--config", "", "diag"])
    assert result.exit_code == 0



import pathlib
import sys

from odoo_runbot.runbot_config import (
    RunbotExcludeWarning,
    RunbotStepConfig,
    RunbotToolConfig,
    StepAction,
    log_filter_by_odoo_version,
    warning_filter_by_odoo_version,
)

if sys.version_info >= (3, 11):
    pass
else:
    pass


def sample_config(fname: str) -> RunbotToolConfig:
    path = pathlib.Path(__file__).resolve().parent.joinpath("sample_config", fname)
    return RunbotToolConfig.load_from_file(path)


def minimal_config_test() -> RunbotToolConfig:
    """[tool.runbot]
    modules = ["module_to_test"]
    """
    global_module = ["module_to_test"]
    log_filters = log_filter_by_odoo_version()
    warning_filters = warning_filter_by_odoo_version()
    return RunbotToolConfig(
        steps=[
            RunbotStepConfig(
                include_current_project=True,
                name="default",
                modules=global_module,
                action=StepAction.TESTS,
                test_tags=[],
                coverage=True,
                log_filters=log_filters,
            ),
        ],
        warning_filters=warning_filters,
        pretty=True,
    )


def classic_file_config_test() -> RunbotToolConfig:
    log_filters = log_filter_by_odoo_version()
    warning_filters = warning_filter_by_odoo_version()
    return RunbotToolConfig(
        warning_filters=warning_filters,
        steps=[
            RunbotStepConfig(
                include_current_project=True,
                name="install",
                modules=["moduleA", "moduleB"],
                action=StepAction.INSTALL,
                test_tags=[],
                coverage=False,
                log_filters=log_filters,
            ),
            RunbotStepConfig(
                include_current_project=True,
                name="step1",
                modules=["module3"],
                action=StepAction.TESTS,
                test_tags=[],
                coverage=True,
                log_filters=log_filters,
            ),
            RunbotStepConfig(
                include_current_project=True,
                name="step2",
                modules=["module_step2"],
                action=StepAction.TESTS,
                test_tags=[],
                coverage=True,
                log_filters=log_filters,
            ),
            RunbotStepConfig(
                include_current_project=True,
                name="step3",
                modules=["moduleA", "moduleB"],
                action=StepAction.TESTS,
                test_tags=[],
                coverage=True,
                log_filters=log_filters,
            ),
        ],
        pretty=True,
    )


def pyproject_classic_test() -> RunbotToolConfig:
    global_module = ["module_to_test", "module_to_test_2"]
    odoo_filters = log_filter_by_odoo_version()
    global_log_filter = [
        RunbotExcludeWarning(
            name=f"All Steps - Logger Filter {len(odoo_filters) + 1}",
            regex=".*log to accept.*",
            logger="",
            min_match=1,
            max_match=1,
        )
    ]

    warning_filters = warning_filter_by_odoo_version()
    return RunbotToolConfig(
        warning_filters=warning_filters,
        steps=[
            RunbotStepConfig(
                include_current_project=True,
                name="install",
                modules=global_module,
                action=StepAction.INSTALL,
                test_tags=[],
                coverage=False,
                log_filters=odoo_filters + global_log_filter,
            ),
            RunbotStepConfig(
                include_current_project=True,
                name="tests",
                modules=global_module,
                action=StepAction.TESTS,
                test_tags=["/module_to_test:MyTestCase", "/module_to_test"],
                coverage=True,
                log_filters=odoo_filters + global_log_filter,
            ),
        ],
        pretty=True,
    )


def pyproject_complex_test() -> RunbotToolConfig:
    odoo_filters = log_filter_by_odoo_version()

    warning_filters = warning_filter_by_odoo_version()
    global_regex = [
        RunbotExcludeWarning(
            name=f"All Steps - Logger Filter {len(odoo_filters) + 1}",
            regex=r".*global-regex-warning-1.*",
        ),
        RunbotExcludeWarning(
            name="global-regex-warning-2",
            regex=r".*global-regex-warning-2.*",
            min_match=1,
            max_match=2,
        ),
    ]
    global_module = ["module_to_test", "module_to_test2"]
    global_coverage = False

    return RunbotToolConfig(
        warning_filters=warning_filters,
        steps=[
            RunbotStepConfig(
                include_current_project=True,
                name="install",
                modules=["first_module_to_install"],
                action=StepAction.INSTALL,
                test_tags=[],
                coverage=global_coverage,
                log_filters=odoo_filters
                + global_regex
                + [
                    RunbotExcludeWarning(
                        regex=".*Install filter.*",
                        name=f"Step install - Logger Filter {len(odoo_filters) + 3}",
                        min_match=1,
                        max_match=1,
                    ),
                ],
            ),
            RunbotStepConfig(
                include_current_project=True,
                name="tests",
                modules=global_module,
                action=StepAction.TESTS,
                test_tags=["+at-install", "-post-install"],
                coverage=True,
                log_filters=odoo_filters
                + global_regex
                + [
                    RunbotExcludeWarning(
                        regex=".*regex warning.*",
                        name="test-regex-log-warning-2",
                        min_match=1,
                        max_match=1,
                    ),
                ],
            ),
            RunbotStepConfig(
                include_current_project=True,
                name="warmup",
                modules=["second_module_to_install"],
                action=StepAction.INSTALL,
                test_tags=[],
                coverage=global_coverage,
                log_filters=odoo_filters + global_regex,
            ),
            RunbotStepConfig(
                include_current_project=True,
                name="Post install test",
                modules=["module_to_test2"],
                action=StepAction.TESTS,
                test_tags=["-at-install", "+post-install"],
                coverage=False,
                log_filters=odoo_filters
                + global_regex
                + [
                    RunbotExcludeWarning(
                        name=f"Step Post install test - Logger Filter {len(odoo_filters) + 3}",
                        regex=".*Post install test regex-warnings.*",
                        min_match=2,
                        max_match=2,
                    ),
                ],
            ),
        ],
        pretty=True,
    )
